<?php
/**
 * @package		Advanced Tags & Prefixes
 * @author		Ryan Hoerr
 * @copyright	(c) 2012 Ryan Hoerr / Sublime Development
 * @license		http://www.sublimism.com/products/terms-of-use
 * @version		$Id: tools.php 37 2013-05-16 03:08:24Z No1_1000 $
 * @updated		$Date: 2013-05-15 23:08:24 -0400 (Wed, 15 May 2013) $
 */

/**
 * Do all the other ACP Prefix management actions and views
 */

if ( ! defined( 'IN_ACP' ) ) {
	print "<h1>Incorrect access</h1>You cannot access this file directly. If you have recently upgraded, make sure you upgraded 'admin.php'.";
	exit();
}

class admin_advancedtagsprefixes_tags_tools extends ipsCommand
{
	private $html;
	private $form_code;
	private $form_code_js;
	private $han_editor;

	public function doExecute( ipsRegistry $registry ) 
	{
		//-----------------------------------------
		// Load skin
		//-----------------------------------------
		
		$this->html			= $this->registry->output->loadTemplate('cp_skin_tags');
		
		//-----------------------------------------
		// Set up stuff
		//-----------------------------------------

		$this->form_code	= $this->html->form_code	= 'module=tags&amp;section=tools';
		$this->form_code_js	= $this->html->form_code_js	= 'module=tags&section=tools';
		
		$classToLoad = IPSLib::loadLibrary( IPSLib::getAppDir( 'advancedtagsprefixes' ) . "/sources/classes/common.php", 'sldTopicPrefixes_common', 'advancedtagsprefixes' );
		$this->common = new $classToLoad( $registry );

		//-----------------------------------------
		// Load lang
		//-----------------------------------------
		
		$this->registry->class_localization->loadLanguageFile( array( 'admin_global' ), 'advancedtagsprefixes' );

		///-----------------------------------------
		// What to do...
		//------------------------------------------
		
		$this->prefixes = $this->cache->getCache('topic_prefixes');
		
		switch( $this->request['do'] )
		{
			default:
			case 'tagManager':
				$this->_showTagMgr();
			break;

			case 'updateTags':
				$this->_updateTags();
			break;

			case 'deleteTags':
				$this->_deleteTags();
			break;
		}
		
		//-----------------------------------------
		// Pass to CP output hander
		//-----------------------------------------
		
		$this->registry->output->html_main .= $this->registry->output->global_template->global_frame_wrapper();
		$this->registry->output->sendOutput();
		
	}

	/**
	 * Show tag list/cloud display for easy management
	 */
	protected function _showTagMgr()
	{
		// Check for/create index
		if( $this->DB->checkForIndex( 'tag_mgr', 'core_tags' ) == false ) {
			$this->DB->addIndex( 'core_tags', 'tag_mgr', 'tag_text(100), tag_meta_app(20)' );
		}

		// Set up sorting/searching/pagination
		$keys	= array( 'tag_text', 'tag_meta_app', 'num' );
		$sort	= in_array($this->request['sort'], $keys) ? $this->request['sort'] : 'tag_text';
		$search = in_array($this->request['search_key'], $keys) ? $this->request['search_key'] : 'tag_text';
		$dir	= $this->request['dir'] == 'desc' ? 'desc' : 'asc';
		$offset	= intval($this->request['st']);
		$limit	= 25;
		$where	= !empty($this->request['tag_search']) ? "{$search} like '%" . IPSText::safeslashes($this->request['tag_search']) . "%'" : '';
		
		$count	= $this->DB->buildAndFetch( array(	'select'	=> 'count(distinct tag_text, tag_meta_app) as total',
													'from'		=> 'core_tags',
													'where'		=> $where ) );

		// Fetch entries
		$this->DB->build( array(	'select'	=> 'tag_text, tag_meta_app, count(*) as num',
									'from'		=> 'core_tags',
									'group'		=> 'tag_text, tag_meta_app',
									'where'		=> $where,
									'limit'		=> array( $offset, $limit ),
									'order'		=> $sort . ' ' . $dir ) );
		$this->DB->execute();

		while( $r = $this->DB->fetch() ) {
			$tags[] = $r;
		}

		if( count($tags) < $count['total'] ) {
			$pagination = $this->registry->output->generatePagination(
				array(	'totalItems'	=> $count['total'],
						'itemsPerPage'	=> $limit, 
						'dotsSkip'		=> 5,
						'startValueKey'	=> 'st',
						'currentStartValue' => $offset,
						'baseUrl'		=> $this->settings['base_url'] . $this->form_code_js . '&do=tagManager&dir=' . $dir . '&sort=' . $sort . '&st=' . $offset . '&search_key=' . $search . '&tag_search=' . urlencode($this->request['tag_search'])
				) );
		}
		else {
			$pagination = '';
		}

		$this->registry->output->html .= $this->html->tagList( $tags, $pagination );
	}

	/**
	 * Update tags matching something to be something else.
	 */
	protected function _updateTags()
	{
		$find			= IPSText::safeslashes( $this->request['find'] );
		$replace_text	= IPSText::safeslashes( $this->request['replace_text'] );
		$app 			= IPSText::safeslashes( $this->request['a'] );
		$replace		= intval( $this->request['replace'] );

		if( empty($replace_text) && $replace > 0 ) {
			$replace_text = $this->prefixes[ $replace ]['prefix_title'];
		}

		if( empty($replace_text) || empty($find) ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_missing_data'] );
		}

		$count = $this->common->updateTags( $find, $replace_text, $app );

		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_massupdate_log'], $count, $find, $replace_text ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_massupdate_resp'], $count ) );
	}

	/**
	 * Remove tags matching something.
	 */
	protected function _deleteTags()
	{
		$find	= $this->request['find'];
		$app	= IPSText::safeslashes( $this->request['a'] );

		if( empty($find) ) {
			$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, $this->lang->words['pre_missing_data'] );
		}

		$count	= $this->common->deleteTags( $find, $app );

		$this->registry->adminFunctions->saveAdminLog( sprintf( $this->lang->words['pre_massdelete_log'], $count, $find ) );
		$this->registry->output->redirect( $this->settings['base_url'] . $this->form_code, sprintf( $this->lang->words['pre_massdelete_resp'], $count, $find ) );
	}
}
