<?php
/**
 * @package		Advanced Tags & Prefixes
 * @author		Ryan Hoerr
 * @copyright	(c) 2012 Ryan Hoerr / Sublime Development
 * @license		http://www.sublimism.com/products/terms-of-use
 * @version		$Id: common.php 35 2012-12-11 18:06:43Z No1 1000 $
 * @updated		$Date: 2012-12-11 13:06:43 -0500 (Tue, 11 Dec 2012) $
 */

class sldTopicPrefixes_common
{
	protected $registry;
	protected $DB;
	protected $settings;
	protected $request;
	protected $lang;
	protected $member;
	protected $memberData;
	protected $cache;
	protected $caches;
	protected $prefixes;

	public function __construct( ipsRegistry $registry )
	{
		/* Make objects */
		$this->registry = $registry;
		$this->DB	    = $this->registry->DB();
		$this->settings =& $this->registry->fetchSettings();
		$this->request  =& $this->registry->fetchRequest();
		$this->lang	    = $this->registry->getClass('class_localization');
		$this->member   = $this->registry->member();
		$this->memberData =& $this->registry->member()->fetchMemberData();
		$this->cache	= $this->registry->cache();
		$this->caches   =& $this->registry->cache()->fetchCaches();
		$this->prefixes = $this->cache->getCache('topic_prefixes');
	}

	/**
	 * Check for common errors and misconfigurations.
	 */
	public function checkState()
	{
		$warnings = array();

		/**
		 * Check for legacy files
		 */
		$files = array( IPSLib::getAppDir( 'forums' ) . '/extensions/admin/forum_form.php',
						IPSLib::getAppDir( 'forums' ) . '/modules_admin/prefixes/' );
		foreach( $files as $file ) {
			if( file_exists( $file ) ) {
				$warnings[] = sprintf( $this->lang->words['pre_legacy_file'], $file );
			}
		}
		if( count($warnings) ) {
			$warnings[] = $this->lang->words['pre_legacy_warn_post'];
		}

		/**
		 * Check for the hook
		 */
		$hook = $this->DB->buildAndFetch( array(	'select'	=> '*',
													'from'		=> 'core_hooks',
													'where'		=> "hook_key='sld_topic_prefixes'" ) );
		if( !count($hook) ) {
			$warnings[] = "Advanced Tags &amp; Prefixes hook is <b>not installed</b>! This application will not function properly without it. Please seek support.";
		}
		elseif( !$hook['hook_enabled'] ) {
			$warnings[] = "Advanced Tags &amp; Prefixes hook is <b>disabled</b>. It must be enabled for this application to function properly. Go to the <a href='".$this->settings['_base_url']."app=core&amp;module=applications&amp;section=hooks&amp;do=hooks_overview'>Hook Manager</a> to enable it.";
		}

		/**
		 * Check settings
		 */
		$tagGroup = $this->DB->buildAndFetch( array(	'select'	=> 'conf_title_id',
														'from'		=> 'core_sys_settings_titles',
														'where'		=> "conf_title_keyword='globaltags'" ) );
		if( $this->settings['tags_enabled'] == 0 ) {
			$warnings[] = "Possible configuration error: <b>Tags are disabled.</b> Tags and prefixes can not be used in this state. To change this, go to the <a href='".$this->settings['_base_url']."app=core&module=settings&section=settings&do=setting_view&conf_group=".$tagGroup['conf_title_id']."'>Tag Settings Group</a> and enable tags.";
		}

		if( $this->settings['tags_can_prefix'] == 1 ) {
			$warnings[] = "Possible configuration error: <b>Only administrators can set prefixes.</b> To change this, go to the <a href='".$this->settings['_base_url']."app=core&module=settings&section=settings&do=setting_view&conf_group=".$tagGroup['conf_title_id']."'>Tag Settings Group</a> and change 'Can use a tag as content prefix'.";
		}
		else if( $this->settings['tags_can_prefix'] == 0 ) {
			$warnings[] = "Possible configuration error: <b>Prefixes are disabled.</b> To change this, go to the <a href='".$this->settings['_base_url']."app=core&module=settings&section=settings&do=setting_view&conf_group=".$tagGroup['conf_title_id']."'>Tag Settings Group</a> and change 'Can use a tag as content prefix'.";
		}

		return $warnings;
	}

	/**
	 * Change all tags and prefixes matching $from to $to.
	 */
	public function updateTags( $from, $to, $app='' )
	{
		$count = 0;
		
		$app_where = !empty($app) ? " and tag_meta_app='" . $app . "'" : '';

		if( $this->settings['tags_force_lower'] ) {
			$this->DB->update( 'core_tags', array( 'tag_text' => IPSText::mbstrtolower( $to ) ), "tag_text='" . IPSText::mbstrtolower( IPSText::safeslashes( $from ) ) . "'" . $app_where );
		}
		else {
			$this->DB->update( 'core_tags', array( 'tag_text' => $to ), "tag_text='" . IPSText::safeslashes( $from ) . "'" . $app_where );
		}

		/**
		 * Update the cache as well.
		 */
		$toRemove	= IPSText::mbstrtolower( $from );
		$toAdd		= $this->settings['tags_force_lower'] ? IPSText::mbstrtolower( $to ) : $to;
		$this->DB->build( array(	'select'	=> 'c.*',
									'from'		=> array( 'core_tags_cache' => 'c' ),
									'where'		=> "tag_cache_text like '%" . IPSText::safeslashes( $toRemove ) . "%'" . $app_where,
									'group'		=> 'tag_cache_key',
									'add_join'	=> array( array(
												'select'	=> 'tag_meta_app',
												'from'		=> array('core_tags' => 't'),
												'type'		=> 'left',
												'where'		=> 't.tag_aai_lookup=c.tag_cache_key',
											) )
										) );
		$a = $this->DB->execute();
		while( $r = $this->DB->fetch( $a ) ) {
			$tags = unserialize($r['tag_cache_text']);
			foreach( $tags['tags'] as $k => $tag ) {
				if( IPSText::mbstrtolower( $tag ) == $toRemove ) {
					$tags['tags'][ $k ] = $toAdd;
				}
			}

			if( IPSText::mbstrtolower( $tags['prefix'] ) == $toRemove ) {
				$tags['prefix'] = $toAdd;
			}
			
			$this->DB->update( 'core_tags_cache', array( 'tag_cache_text' => serialize($tags) ), "tag_cache_key='" . $r['tag_cache_key'] . "'" );
			$count++;
		}

		return $count;
	}

	/**
	 * Remove all tags and prefixes matching $find.
	 */
	public function deleteTags( $find, $app='' )
	{
		$app_where = !empty($app) ? " and tag_meta_app='" . $app . "'" : '';

		$find  = IPSText::safeslashes( $find );
		$this->DB->delete( 'core_tags', "tag_text='" . $find . "'" . $app_where );
		$count = $this->DB->getAffectedRows();

		/**
		 * Update the cache as well.
		 */
		$toRemove	= IPSText::mbstrtolower( $find );
		$this->DB->build( array(	'select'	=> 'c.*',
									'from'		=> array( 'core_tags_cache' => 'c' ),
									'where'		=> "tag_cache_text like '%" . $toRemove . "%'" . $app_where,
									'group'		=> 'tag_cache_key',
									'add_join'	=> array( array(
												'select'	=> 'tag_meta_app',
												'from'		=> array('core_tags' => 't'),
												'type'		=> 'left',
												'where'		=> 't.tag_aai_lookup=c.tag_cache_key',
											) )
										) );
		$a = $this->DB->execute();
		while( $r = $this->DB->fetch( $a ) ) {
			$tags = unserialize($r['tag_cache_text']);
			foreach( $tags['tags'] as $k => $tag ) {
				if( IPSText::mbstrtolower( $tag ) == $toRemove ) {
					unset( $tags['tags'][ $k ] );
				}
			}

			if( IPSText::mbstrtolower( $tags['prefix'] ) == $toRemove ) {
				$tags['prefix'] = '';
			}
			
			$this->DB->update( 'core_tags_cache', array( 'tag_cache_text' => serialize($tags) ), "tag_cache_key='" . $r['tag_cache_key'] . "'" );
		}

		return $count;
	}
	
	/**
	 * Remove the given tag from all forums with lots of wizardry.
	 */
	public function _removeFromForums( $toRemove )
	{
		$toRemove = IPSText::mbstrtolower( $toRemove );
		
		$this->DB->build( array(	'select'	=> 'id, tag_predefined',
									'from'		=> 'forums',
									'where'		=> "tag_predefined like '%" . $toRemove . "%'" ) );
		$a = $this->DB->execute();
		while( $r = $this->DB->fetch( $a ) ) {
			$tags = array_filter( explode( ',', $r['tag_predefined'] ) );
			foreach( $tags as $k => $tag ) {
				if( IPSText::mbstrtolower( trim( $tag ) ) == $toRemove ) {
					unset( $tags[ $k ] );
				}
			}
			
			$this->DB->update( 'forums', array( 'tag_predefined' => IPSText::safeslashes( implode( ',', $tags ) ) ), 'id=' . $r['id'] );
		}
		
		return;
	}
	
	/**
	 * Add the given tag to the given forums with lots of wizardry.
	 */
	public function _addToForums( $toAdd, $ids )
	{
		if( count( $ids ) ) {
			foreach( $ids as $forum ) {
				if( intval($forum) > 0 )
					$forums[] = intval($forum);
			}
			
			$this->DB->build( array(	'select'	=> 'id, tag_predefined',
										'from'		=> 'forums',
										'where'		=> 'id in(' . implode( ',', $forums ) . ')' ) );
			$a = $this->DB->execute();
			while( $r = $this->DB->fetch( $a ) ) {
				// Check whether it's already there among any existing tags.
				$tags = array_filter( explode( ',', $r['tag_predefined'] ) );
				if( count( $tags ) ) {
					foreach( $tags as $tag ) {
						if( trim( $tag ) == $toAdd ) {
							continue 2;
						}
					}
				}
				
				// Not found, so add to the list and insert the prefix.
				$tags[] = $toAdd;
				$this->DB->build( array(	'update'	=> 'forums',
											'set'		=> "tag_predefined='" . IPSText::safeslashes( implode( ',', $tags ) ) . "'",
											'where'		=> 'id=' . $r['id'] ) );
				$this->DB->execute();
			}
		}
		
		return;
	}

	/**
	 * Clean prefixes in accordance with classes_tag_abstract->_stripHtml().
	 */
	public function _cleanPrefix( $prefix )
	{
		return str_replace( array( '&#62;', '&#60;', '&amp;', '&quot;', '&#39;', '<', '>', '"', "'", "," ), '', $prefix );
	}
}
